// RW Bumpmaster Project
// =====================
// Imageboard downloader + archiver demo project;  SAVE.EVERYTHING.ANON. :^)

// Filename: ./main_bumpmaster.cpp
// -The program's entry point

#include "incls_all.hpp"

//------------------------------------------------------------------------------
/** Bumpmaster test; print various board's Page 1 thread's 5 latest posts
 * @note: this is similar to an IB index page (as a text-only printout)
 */
int main() try {
  puts("\n SAVE.EVERYTHING.ANON. :^)");

  Bumpmaster bump{"bumpmaster"};

  auto const thrd_lim{10};  // just the board's Page 1 threads
  auto const post_lim{5};   // just the 5 latest posts each

  // container of various IB URI strings
  vector<string> const board_uris{
      // NOTE: currently supporting only Lynxchan sites (for v220226)

      "https://alogs.space/robowaifu/",
      "https://prolikewoah.com/geimu/",
      "https://late.city/late/",
      "https://anon.cafe/christian/",

  };

  // cycle the board URI strings (& print their data)
  for (auto const& board_uri : board_uris) {
    // build the catalog's JSON URI
    auto const& [site, board] = bump.parse_ib_uri(board_uri);
    auto const cat_uri        = bump.bld_cat_uri(site, board);

    puts("\n\n     --=^=--\n\n");

    cout << " =======================================\n"
         << "    Pulling /" << board << "/'s dolldrops...\n"
         << " =======================================\n";

    // pull catalog's dolldrop
    cout << "\n page_uri:    " << cat_uri << '\n';
    auto const cat_json = bump.pull_json_drop(cat_uri);

    // load new container with catalog's Page 1 threads + posts; in bump-order
    auto const top_thrds = bump.top_bump_thrds(cat_json, thrd_lim, site, board);

    // cycle the top threads (& print their data)
    size_t t_idx{0};                   //
    for (auto const& t : top_thrds) {  //
      puts("\n\n     --=^=--\n");

      cout << "     Bump  " << ++t_idx                        // prt thrd's dat
           << "\n =======================================\n"  //
           << setw(8) << setfill(' ') << t.id << "  :  "      //
           << t.subj << "\n\n' " << t.msg << " '\n";          //

      // calc starting post's thread index
      auto const sz = static_cast<int>(t.posts.size());  // cast avoids overflow
      auto const start{sz - post_lim};                   // for this subtraction

      // cycle the latest posts (& print their data)
      for (auto p_idx{(start >= 0 ? start : 0)};  // ensure we have valid index
           p_idx < sz; ++p_idx) {                 // (as cnt may be < post_lim)

        auto const& p = t.posts[p_idx];  // index to post
        cout << '\n'                     // prt post's dat
             << setw(33) << setfill(' ') << "r " << (p_idx + 1)
             << "\n  ------------------------------------\n"
             << setw(8) << setfill(' ') << p.id << "  :  " << p.date << "\n' "
             << p.msg << " '\n";
      }
    }
  }

  puts("\n\n     --=^=--\n\n");

  puts(" ...leaving main()\n");

} catch (runtime_error const& e) {
  cerr << "\n  runtime_error '" << e.what() << "' caught in main()\n\n";

} catch (exception const& e) {
  cerr << "\n  exception '" << e.what() << "' caught in main()\n\n";

} catch (...) {
  cerr << "\n  unknown error caught in main()\n\n";
}

//    --=^=--

// "Everyone's talking about IT & the Internet, and how advanced it's getting."

// rw_bumpmaster_demo.v220226
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
