//  RW Dolldrop
// ==============
// External File Transfers Library
// -Part of the RW Foundations suite of libraries

// Filename: ./src/dollhouse/net/dolldrop.cpp
// -Definitions of the baseline RW Dolldrop sub-library

///////
// Dolldrop/Dollnet is meant for network-enabled RW projects only
#if __has_include(<curl_easy.h>)

#include "incls_all.hpp"

namespace rw::dollhouse::net {

//------------------------------------------------------------------------------
Dolldrop_b::Dolldrop_b(Hip const& tag, std::string const& handle)
    : RW_slave{{Type::Bumpmaster, Type::Curator}, handle, Type::Dolldrop} {
  stamp("Dolldrop_b()");

  bind_master(tag);
}

//------------------------------------------------------------------------------
auto Dolldrop_b::chkd_pull_g(Hip const& tag, std::string const& uri)
    -> std::fstream {
  stamp(tag, "chkd_pull_g()");

  // first, pull the dolldrop's header
  auto hdr = pull_hdr_g(master(), uri);

  // next, check if the dolldrop is a new one
  const auto& [have_new_drop, name] = chk_droptimes(uri, hdr);

  if (have_new_drop) {
    // adjust the display cursor back up one line (re: prelim header request)
    cout << "\u008D\r" << flush;

    return pull_body_g(master(), uri);

  } else {  // just return the already-saved (and current) dolldrop file

    // TEST:
    // cout << "\n  -local dolldrop is current :  " << name << "\n\n";
    // puts("");
    return fstream{name};
  }
}

//------------------------------------------------------------------------------
auto Dolldrop_b::pull_hdr_g(Hip const& tag, std::string const& uri)
    -> std::stringstream {
  stamp(tag, "pull_hdr_g()");

  // TEST:
  // cout << "  -requested pull address :  " << uri << "\n\n";
  // puts("");

  if (tag == master()) {
    return mailstop_.try_hdr(t(), uri);

  } else
    only_master_can(master(), tag, t(), "Dolldrop_b::pull_hdr_g()");

  ///
  return stringstream{};  // shouldn't reach here
}

//------------------------------------------------------------------------------
auto Dolldrop_b::pull_body_g(Hip const& tag, std::string const& uri)
    -> std::fstream {
  stamp(tag, "pull_body_g()");

  // TEST:
  // prt_tag(tag, "asked to pull a dolldrop");

  if (tag == master()) {
    return mailstop_.try_body(t(), uri);

  } else
    only_master_can(master(), tag, t(), "Dolldrop_b::pull_body_g()");

  ///
  return fstream{};  // shouldn't reach here
}

//------------------------------------------------------------------------------
auto Dolldrop_b::pull_hdr_body_g(Hip const& tag, std::string const& uri)
    -> std::stringstream {
  stamp(tag, "pull_hdr_body_g()");

  // TEST:
  prt_tag(tag, "asked to pull a dolldrop w/ hdr");
  cout << "  -requested pull address :  " << uri << "\n\n";

  if (tag == master()) {
    return mailstop_.try_hdr_body(t(), uri);

  } else
    only_master_can(master(), tag, t(), "Dolldrop_b::pull_hdr_body_g()");

  ///
  return stringstream{};  // shouldn't reach here
}

}  // namespace rw::dollhouse::net

#endif  // __has_include(<curl_easy.h>)

//    --=^=--

// "______"

// rw_dolldrop_lib.v220226
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
